
import time 
from epics import caget,caput,PV

import iexcode.instruments.cfg as iex
from iexcode.instruments.AD_utilities import *
from iexcode.instruments.IEX_cameras import cam_pv_dictionary
from iexcode.macros.quick_plot import *


##############################################################################################################
##############################             Camera Snapping an image         ##############################
##############################################################################################################

def cam_snap(cam_num,ADtype='TIFF',**kwargs):
    """
    takes a camera image and saves it in the user folder
    
    **kwargs:
    ExposureTime: changes both the exposure time and the acquire time for the snapshot
                    resets after acquisition
    FreeRun: True => disable setting and go back to continuous acquision 
                False => leave saving enabled and camera in single acquision

    """
    ADplugin = cam_pv_dictionary()[cam_num]+ADtype+"1:"
    AD_snap(ADplugin,**kwargs)




##############################################################################################################
##############################             Scanning with a camera           ##############################
##############################################################################################################
def cam_scan_setup(cam_num,ADtype='TIFF',**kwargs):
    """
    sets up the BL scanRecord for scanning at every point
    **kwargs
        scanDIM = 1
        detTrig = 2; detectorTrigger number
        
        # AD_SaveFileSetup
        filepath=userpath (from BL_ioc scanRecord)+"/dtype"
         (e.g. filepath="/net/s29data/export/data_29id"+folder+"/"+run+"/"+userName+"/"+df)
        dtype = taken from ADplugin
        FileTemplate="%s%s_%4.4d."+dtype; format for filename first %s = filepath, second %s = prefix
        prefix = dtype by default
    """
    ADplugin = cam_pv_dictionary()[cam_num]+ADtype+"1:"
    ADplugin_ScanSetup(ADplugin,iex.BL.mda, **kwargs)  

def cam_live(cam_num):
    """
    puts camera in no save and continuous
    a.k.a free run

    """
    ADplugin = cam_pv_dictionary()[cam_num]
    AD_FreeRun(ADplugin)

def cam_done(cam_num,**kwargs):
    """
    stops camera acquisition (not will stop after current acquisition)
    """
    ADplugin = cam_pv_dictionary()[cam_num]
    AD_Done(ADplugin,**kwargs)

def cam_save_setup(cam_num,ADtype='TIFF',**kwargs):
    """
    sets up the save paths using iex.BL.mda.current_user
    **kwargs 
        prefix: beginning of filename
    """
    ADplugin = cam_pv_dictionary()[cam_num]+ADtype+"1:"
    AD_SaveFileSetup(ADplugin,**kwargs)

def cam_last_filepath(cam_num,ADtype='TIFF',verbose=True):
    """
    returns the full path for the last saved image
    """
    ADplugin = cam_pv_dictionary()[cam_num]+ADtype+"1:"
    fpath = AD_last_filename(ADplugin)
    return fpath
    
def cam_plot_last(cam_num,ADtype='TIFF',h=5,v=10,**kwargs):
    """
    plots last file from cam_num
    using iexcode.macros.quick_plot
    """
    fpath = cam_last_filepath(cam_num,verbose=2)
    plot_image(fpath,h,v,**kwargs)


